@@ -27,6 +27,16 @@
 --
 --
 -- $Log: backup.c,v $
+-- Revision 1.20  2008/06/21 15:06:19  jsedwards
+-- Added code to log backup file name when opening.
+--
+-- Revision 1.19  2008/06/14 16:13:02  jsedwards
+-- Added a new line character to warning message for existing file.
+--
+-- Revision 1.18  2008/06/14 13:02:12  jsedwards
+-- Fix Bug #1992924 - Changed so that if backup (.dif) file already exists it
+-- will wait one second, generate a new file name and try to open it again.
+--
 -- Revision 1.17  2008/02/04 03:01:17  jsedwards
 -- Added 'const' to backup_location parameter and added a '/' between the
 -- backup_location (directory) and the file name.
@@ -132,6 +142,8 @@ void nwos_initialize_backup(const char* backup_location, Disk_Header* disk_heade
     char* backup_file_suffix = ".dif";
     uint8 buffer[FILE_BLOCK_SIZE];
     size_t bytes_written();
+    int retry_count = 0;
+    char buf[128];
 
     assert(backup_location != NULL);
 
@@ -147,23 +159,39 @@ void nwos_initialize_backup(const char* backup_location, Disk_Header* disk_heade
     username = "objectify";   /* for now just use this instead of the username */
 #endif
     {
-	nwos_get_time_stamp(time_stamp);
-
 	path_length = strlen(backup_location) + 1 + strlen(username) + 1 + strlen(backup_file_suffix) + 14 + 1;
 	nwos_backup_path = malloc(path_length);
 
-	snprintf(nwos_backup_path, path_length, "%s/%s-%d%02d%02d%02d%02d%02d%s",
-		 backup_location,
-		 username,
-		 nwos_extract_year_from_time_stamp(time_stamp),
-		 nwos_extract_month_from_time_stamp(time_stamp),
-		 nwos_extract_day_of_month_from_time_stamp(time_stamp),
-		 nwos_extract_hour_from_time_stamp(time_stamp),
-		 nwos_extract_minute_from_time_stamp(time_stamp),
-		 nwos_extract_second_from_time_stamp(time_stamp),
-		 backup_file_suffix);
+	while (1)
+	{
+	    nwos_get_time_stamp(time_stamp);
 
-	backup_file_desc = open(nwos_backup_path, O_RDWR | O_CREAT | O_EXCL, S_IRUSR | S_IWUSR);
+	    snprintf(nwos_backup_path, path_length, "%s/%s-%d%02d%02d%02d%02d%02d%s",
+		     backup_location,
+		     username,
+		     nwos_extract_year_from_time_stamp(time_stamp),
+		     nwos_extract_month_from_time_stamp(time_stamp),
+		     nwos_extract_day_of_month_from_time_stamp(time_stamp),
+		     nwos_extract_hour_from_time_stamp(time_stamp),
+		     nwos_extract_minute_from_time_stamp(time_stamp),
+		     nwos_extract_second_from_time_stamp(time_stamp),
+		     backup_file_suffix);
+
+	    snprintf(buf, sizeof(buf), "Opening backup file: %s", nwos_backup_path);
+	    nwos_log(buf);
+
+	    errno = 0;
+	    backup_file_desc = open(nwos_backup_path, O_RDWR | O_CREAT | O_EXCL, S_IRUSR | S_IWUSR);
+
+	    /* exit the loop if a file by that name didn't already exist or we have retried 5 times already */
+	    if (errno != EEXIST || retry_count > 5) break;
+
+	    fprintf(stderr, "Warning: backup file %s already existed.\n", nwos_backup_path);
+
+	    sleep(1);                /* wait one second so that the file name will be different next time */
+
+	    retry_count++;
+	}
 
 	if (backup_file_desc < 0)
 	{
