@@ -27,6 +27,11 @@
 --
 --
 -- $Log: class_definition.c,v $
+-- Revision 1.26  2008/03/15 16:41:00  jsedwards
+-- Changed 'nwos_find_private_class_definition' to call new
+-- 'nwos_get_object_class_without_update function' instead of
+-- 'nwos_get_object_class' to keep from getting in a recursive mess.
+--
 -- Revision 1.25  2007/07/28 01:20:09  jsedwards
 -- Changes to deal with classes that have been updated.
 --
@@ -486,7 +491,8 @@ bool nwos_find_private_class_definition(char* name, ObjRef* class_ref)
 
     for (i = 0; i < num_classes; i++)
     {
-	nwos_get_object_class(&ref_list->references[i], &object_class);   /* find out what kind of object it is */
+	/* find out what kind of object it is without update to keep from getting into a recursive mess */
+	nwos_get_object_class_without_update(&ref_list->references[i], &object_class);
 
 	if (is_same_object(&object_class, &nwos_private_class_definition_class_ref))   /* it is a class definition object */
 	{
