@@ -27,6 +27,10 @@
 --
 --
 -- $Log: compress_sparse.c,v $
+-- Revision 1.23  2008/04/28 12:48:55  jsedwards
+-- Added code to check for blocks where the reference ID is zero and skip over
+-- them with a warning.
+--
 -- Revision 1.22  2008/03/12 03:56:38  jsedwards
 -- Changed to use the GNU MD5 context and functions instead of the RSA context
 -- and functions.
@@ -163,6 +167,7 @@ int main(int argc, char* argv[])
     uint8 block_map[BIT_MAP_BYTES];
     uint8 block[FILE_BLOCK_SIZE];
     int i;
+    int j;
     size_t bytes_read;
     FILE* ofp = NULL;
     int num_blocks;
@@ -181,6 +186,7 @@ int main(int argc, char* argv[])
     int chunk_num;
     uint32 used_chunks;
     Chunk_Info* chunk_info;
+    int blocks_with_bad_ids = 0;
 
 
     for (i = 0; i < SIZE_COUNTS; i++) counts[i] = 0;
@@ -346,28 +352,51 @@ int main(int argc, char* argv[])
 		
 		ref = (uint32)block[4] << 24 | (uint32)block[5] << 16 | (uint32)block[6] << 8 | (uint32)block[7];
 
-		printf("id: %08x  block: %08x\n", ref, (uint32)chunk+i);
-	     /* printf("id: %08x\n", ref); */
-		fflush(stdout);
-
-		counts[ref >> 28]++;
-
-		if (ofp != NULL && fwrite(block, 1, sizeof(block), ofp) != sizeof(block))
+		if (ref == 0)
 		{
-		    perror(argv[1]);
-		    close(obj_file_desc);
-		    exit(1);
-		}
+		    blocks_with_bad_ids++;
 
-		md5_process_bytes(block, sizeof(block), &md5_context);    /* include this data in the md5 checksum */
-		sha1_process_bytes(block, sizeof(block), &sha1_context);    /* include this data in the sha1 checksum */
-		num_blocks++;
+		    printf("WARNING: reference in block is zero, not written:\n");
+
+		    for (j = 0; j < FILE_BLOCK_SIZE; j += 16)
+		    {
+			printf("%02x%02x%02x%02x %02x%02x%02x%02x %02x%02x%02x%02x %02x%02x%02x%02x\n",
+			       block[j+0],  block[j+1],  block[j+2],  block[j+3],
+			       block[j+4],  block[j+5],  block[j+6],  block[j+7],
+			       block[j+8],  block[j+9],  block[j+10], block[j+11],
+			       block[j+12], block[j+13], block[j+14], block[j+15]);
+		      }
+		}
+		else
+		{
+		    printf("id: %08x  block: %08x\n", ref, (uint32)chunk+i);
+		    /* printf("id: %08x\n", ref); */
+		    fflush(stdout);
+
+		    counts[ref >> 28]++;
+
+		    if (ofp != NULL && fwrite(block, 1, sizeof(block), ofp) != sizeof(block))
+		    {
+			perror(argv[1]);
+			close(obj_file_desc);
+			exit(1);
+		    }
+
+		    md5_process_bytes(block, sizeof(block), &md5_context);    /* include this data in the md5 checksum */
+		    sha1_process_bytes(block, sizeof(block), &sha1_context);    /* include this data in the sha1 checksum */
+		    num_blocks++;
+		}
 	    }
 	}
     }
 
     printf("Number of blocks: %d\n", num_blocks);
 
+    if (blocks_with_bad_ids > 0)
+    {
+	printf("WARNING: %d blocks had a reference ID of zero and weren't written!\n", blocks_with_bad_ids);
+    }
+
     md5_finish_ctx(&md5_context, md5_digest);   /* finish computing the md5 sum */
     sha1_finish_ctx(&sha1_context, sha1_digest);
 
