@@ -27,6 +27,18 @@
 --
 --
 -- $Log: disk_io.c,v $
+-- Revision 1.84  2008/06/12 02:16:29  jsedwards
+-- Fixed so that read_only flag is not set in PUBLIC_MODE, because if it is
+-- set no backup file gets written.
+--
+-- Revision 1.83  2008/06/07 16:26:21  jsedwards
+-- Fix Bug #1987367 - moved setting of 'read_only' variable outside of if
+-- private objects statement so that it is always set, even if there are no
+-- private objects.
+--
+-- Revision 1.82  2008/05/22 11:33:51  jsedwards
+-- Made disk_header global and renamed to nwos_disk_header.
+--
 -- Revision 1.81  2008/05/05 12:41:15  jsedwards
 -- Changed so that if fopen of index file fails that it just prints a warning
 -- and continues without writing the index file.
@@ -352,7 +364,7 @@
 #include "objectify_private.h"
 
 
-static Disk_Header disk_header;
+Disk_Header nwos_disk_header;  /* this should NOT be accessed by normal programs */
 
 const char*  nwos_public_path;
 const char*  nwos_private_path;
@@ -711,32 +723,36 @@ void nwos_initialize_disk_io(AccessType type, const char* path)
     }
 #endif
 
-    if (read(public_file_desc, &disk_header, sizeof(disk_header)) != sizeof(disk_header))
+    if (read(public_file_desc, &nwos_disk_header, sizeof(nwos_disk_header)) != sizeof(nwos_disk_header))
     {
 	snprintf(log_msg, sizeof(log_msg), "reading disk header from: %s", nwos_public_path);
 	perror(log_msg);
 	exit(1);
     }
 
-    if (memcmp(disk_header.magic_number, MAGIC_NUMBER, 4) != 0)
+    if (memcmp(nwos_disk_header.magic_number, MAGIC_NUMBER, 4) != 0)
     {
 	fprintf(stderr, "Missing magic number in disk header\n");
 	exit(1);
     }
 
-    if (memcmp(disk_header.version_string, VERSION_STRING, 4) != 0)
+    if (memcmp(nwos_disk_header.version_string, VERSION_STRING, 4) != 0)
     {
 	fprintf(stderr, "Incorrect version string in disk header\n");
 	exit(1);
     }
 
-    nwos_4_uint8_to_uint32(disk_header.used_blocks, &nwos_used_public_blocks);
+    nwos_4_uint8_to_uint32(nwos_disk_header.used_blocks, &nwos_used_public_blocks);
 
     snprintf(log_msg, sizeof(log_msg), " used public blocks: %9u", nwos_used_public_blocks);
     nwos_log(log_msg);
 
 #ifdef PUBLIC_MODE
     nwos_word_to_ref(nwos_used_public_blocks, &nwos_next_public_ref);
+
+    /* don't set read_only flag if in PUBLIC_MODE */
+#else
+    read_only = (type == PUBLIC || type == READ_ONLY);
 #endif
 
 
@@ -746,8 +762,6 @@ void nwos_initialize_disk_io(AccessType type, const char* path)
 
     if (nwos_private_path != NULL)
     {
-	read_only = (type == PUBLIC || type == READ_ONLY);
-
 	if (read_only)
 	{
 	    private_file_desc = open(nwos_private_path, O_RDONLY);
@@ -786,35 +800,35 @@ void nwos_initialize_disk_io(AccessType type, const char* path)
 	    }
 	}
 
-	if (read(private_file_desc, &disk_header, sizeof(disk_header)) != sizeof(disk_header))
+	if (read(private_file_desc, &nwos_disk_header, sizeof(nwos_disk_header)) != sizeof(nwos_disk_header))
 	{
 	    snprintf(log_msg, sizeof(log_msg), "reading disk header from: %s", nwos_private_path);
 	    perror(log_msg);
 	    exit(1);
 	}
 
-	if (memcmp(disk_header.magic_number, MAGIC_NUMBER, 4) != 0)
+	if (memcmp(nwos_disk_header.magic_number, MAGIC_NUMBER, 4) != 0)
 	{
 	    fprintf(stderr, "Missing magic number in disk header\n");
 	    exit(1);
 	}
 
-	if (memcmp(disk_header.version_string, VERSION_STRING, 4) != 0)
+	if (memcmp(nwos_disk_header.version_string, VERSION_STRING, 4) != 0)
 	{
 	    // allow more compressed files to be other compatible versions
 
-	    if ((compressed_file && memcmp(disk_header.version_string, OLDEST_COMPATIBLE_COMPRESSED_VERSION, 4) < 0) ||
-		(!compressed_file && memcmp(disk_header.version_string, VERSION_STRING, 4) != 0))
+	    if ((compressed_file && memcmp(nwos_disk_header.version_string, OLDEST_COMPATIBLE_COMPRESSED_VERSION, 4) < 0) ||
+		(!compressed_file && memcmp(nwos_disk_header.version_string, VERSION_STRING, 4) != 0))
 	    {
 		fprintf(stderr, "Incorrect version string in disk header\n");
 		exit(1);
 	    }
 	}
 
-	nwos_4_uint8_to_uint32(disk_header.block_offset_to_chunks, &nwos_block_offset_to_chunks);
-	nwos_4_uint8_to_uint32(disk_header.total_blocks, &nwos_total_private_blocks);
-	nwos_4_uint8_to_uint32(disk_header.used_blocks, &nwos_used_private_blocks);
-	nwos_4_uint8_to_uint32(disk_header.used_chunks, &nwos_used_private_chunks);
+	nwos_4_uint8_to_uint32(nwos_disk_header.block_offset_to_chunks, &nwos_block_offset_to_chunks);
+	nwos_4_uint8_to_uint32(nwos_disk_header.total_blocks, &nwos_total_private_blocks);
+	nwos_4_uint8_to_uint32(nwos_disk_header.used_blocks, &nwos_used_private_blocks);
+	nwos_4_uint8_to_uint32(nwos_disk_header.used_chunks, &nwos_used_private_chunks);
 
 	nwos_total_private_chunks = nwos_total_private_blocks / BLOCKS_IN_CHUNK;
 
@@ -926,7 +940,7 @@ void nwos_initialize_disk_io(AccessType type, const char* path)
 
     if (backup_dir != NULL && !read_only)
     {
-	nwos_initialize_backup(backup_dir, &disk_header);
+	nwos_initialize_backup(backup_dir, &nwos_disk_header);
     }
 #endif
 }
@@ -965,16 +979,16 @@ void nwos_terminate_disk_io()
 		 nwos_ref_to_word(&nwos_next_public_ref));
 	nwos_log(log_msg);
 
-	copy_reference((ObjRef*)&disk_header.used_blocks, &nwos_next_public_ref);
+	copy_reference((ObjRef*)&nwos_disk_header.used_blocks, &nwos_next_public_ref);
 
-	nwos_get_time_stamp(disk_header.last_change);
+	nwos_get_time_stamp(nwos_disk_header.last_change);
 #else
-	nwos_uint32_to_4_uint8(&nwos_used_private_blocks, disk_header.used_blocks);
+	nwos_uint32_to_4_uint8(&nwos_used_private_blocks, nwos_disk_header.used_blocks);
 
 	snprintf(log_msg, sizeof(log_msg), "  private blocks: %9u  used: %9u", nwos_total_private_blocks, nwos_used_private_blocks);
 	nwos_log(log_msg);
 
-	nwos_get_time_stamp(disk_header.last_change);
+	nwos_get_time_stamp(nwos_disk_header.last_change);
 
 	if (chunk_info_modified)
 	{
@@ -1004,7 +1018,7 @@ void nwos_terminate_disk_io()
 		exit(1);
 	    }
 
-	    nwos_uint32_to_4_uint8(&nwos_used_private_chunks, disk_header.used_chunks);
+	    nwos_uint32_to_4_uint8(&nwos_used_private_chunks, nwos_disk_header.used_chunks);
 
 	    chunk_info_modified = false;
 	}
@@ -1021,9 +1035,9 @@ void nwos_terminate_disk_io()
 	}
 
 #ifdef PUBLIC_MODE
-	if (write(public_file_desc, &disk_header, sizeof(disk_header)) != sizeof(disk_header))
+	if (write(public_file_desc, &nwos_disk_header, sizeof(nwos_disk_header)) != sizeof(nwos_disk_header))
 #else
-	if (write(private_file_desc, &disk_header, sizeof(disk_header)) != sizeof(disk_header))
+	if (write(private_file_desc, &nwos_disk_header, sizeof(nwos_disk_header)) != sizeof(nwos_disk_header))
 #endif
 	{
 	    perror(nwos_private_path);
@@ -1044,7 +1058,7 @@ void nwos_terminate_disk_io()
     nwos_private_path = NULL;
 
 #ifndef NO_BACKUP
-    nwos_terminate_backup(modified, &disk_header);
+    nwos_terminate_backup(modified, &nwos_disk_header);
 #endif
 
     if (public_file_desc > 0 && close(public_file_desc) < 0)
